/*
MemAccess Library
Copyright (c) 1999-2006 Hai Li, Zeal SoftStudio.
Web: http://www.zealsoftstudio.com (English)
	 http://china.zealsoft.com (Chinese)
E-Mail: support@zealsoft.com
*/
#ifndef MemAccess

#define MemAccess

#ifdef __cplusplus
extern "C" {
#endif

#include <windows.h>

#define V86_To_PhyAddress(seg,ofs)	((ULONG) \
			   (((unsigned long)(seg) << 4) | (unsigned)(ofs)))

// Determine whether the program is running under WinNT.
BOOL APIENTRY maIsWinNT();

// Determine whether the program is running under 64-bit Windows.
BOOL APIENTRY maIsWin64();

// Open the diver.
BOOL APIENTRY maOpenLibrary();

// Close the driver.
BOOL APIENTRY maCloseLibrary();

// Return a byte word at memory location nAddress.
BYTE APIENTRY maPeekB(DWORD nAddress);
// Return a word at memory location nAddress.
WORD APIENTRY maPeekW(DWORD nAddress);
// Return a double word at memory location nAddress.
DWORD APIENTRY maPeekD(DWORD nAddress);

// Store a byte at memory location nAddress.
BOOL APIENTRY maPokeB(DWORD nAddress, WORD nData);
// Store a word at memory location nAddress.
BOOL APIENTRY maPokeW(DWORD nAddress, WORD nData);
// Store a double word at memory location nAddress.
BOOL APIENTRY maPokeD(DWORD nAddress, DWORD dwData);

// Set the registration information.
void APIENTRY maLicenseInfo(LPSTR sUser, long dwKey);

// Error constants
const int ERROR_MA_DRIVER = 2;
const int ERROR_MA_NO_DEVICE_HANDLE = 9996;
const int ERROR_MA_SCM_CANT_CONNECT = 9998;

// Returns a value indicating the last operation is successful or not.
int APIENTRY maGetLastState(char *s);

// Returns the version of MemAccess Library.
WORD APIENTRY maGetVersion();

// Install the service under Windows NT/2000/XP
BOOL APIENTRY maInstallService();
// Remove the service under Windows NT/2000/XP
BOOL APIENTRY maRemoveService();
// Control the service under Windows NT/2000/XP
BOOL APIENTRY maControlService();

// Map the physical address to process address
void* APIENTRY maMapPhysToLinear(DWORD PhsAddr, DWORD dwSize, HANDLE* hMem);

// Unmap the address
BOOL APIENTRY maUnmapPhysicalMemory(HANDLE hMem, PVOID LinearAddr);

typedef struct {
	ULONG BaseAddress;
	ULONG Size;
	ULONG IOType;
} BADDR;

// Get the base address of a PCI device
LONG APIENTRY maGetDeviceBaseAddress(USHORT* VendorID, USHORT* DeviceID, LONG nIndex, BADDR baddrBuffer[6]);

typedef struct _pci_config_header
{
    USHORT  VendorID;
    USHORT  DeviceID;
    USHORT  Command; 
    USHORT  Status;
    UCHAR   RevisionID;
    UCHAR   ProgIf;    
    UCHAR   SubClass;  
    UCHAR   BaseClass; 
    UCHAR   CacheLineSize;
    UCHAR   LatencyTimer; 
    UCHAR   HeaderType;   
    UCHAR   BIST;          
    ULONG   BaseAddresses[6];
	ULONG	CardBusCISPtr;
    USHORT	SubsystemVendorID;
	USHORT	SubsystemID;
	ULONG	ROMBaseAddress;
    ULONG   Reserved2[2];
    UCHAR   InterruptLine; 
    UCHAR   InterruptPin;  
    UCHAR   MinimumGrant;  
    UCHAR   MaximumLatency;
} PCI_CONFIG_HEADER;

// Get a device information of a PCI device.
BOOL APIENTRY maGetPCIDeviceInfo(USHORT* VendorID, USHORT* DeviceID, LONG nIndex, ULONG* Bus, ULONG* Device, ULONG* Function, PCI_CONFIG_HEADER *pcfg);

// Allocate system buffer
LONG APIENTRY maAllocSystemBuffer(LONG size, LONG * physAddr);

// Free system buffer allocated by maAllocSystemBuffer
BOOL APIENTRY maFreeSystemBuffer();

#ifdef __cplusplus
}
#endif

#endif
